@@ -11,22 +11,21 @@ from typing import Union
 import dns
 from pyleri import (
     Grammar,
+    List,
     Regex,
     Sequence,
-    List,
 )
 
-
-from checkdmarc.utils import (
-    WSP_REGEX,
-    query_dns,
-    normalize_domain,
-    get_base_domain,
-    MAILTO_REGEX,
-    DNSException,
-)
-from checkdmarc.utils import get_mx_records
 from checkdmarc._constants import SYNTAX_ERROR_MARKER
+from checkdmarc.utils import (
+    MAILTO_REGEX,
+    WSP_REGEX,
+    DNSException,
+    get_base_domain,
+    get_mx_records,
+    normalize_domain,
+    query_dns,
+)
 
 """Copyright 2019-2023 Sean Whalen
 
@@ -127,7 +126,7 @@ class UnverifiedDMARCURIDestination(_DMARCWarning):
 
 class MultipleDMARCRecords(DMARCError):
     """Raised when multiple DMARC records are found, in violation of
-    RFC 7486, section 6.6.3"""
+    RFC 7486, § 6.6.3"""
 
 
 class _DMARCGrammar(Grammar):
@@ -166,7 +165,7 @@ dmarc_tags = OrderedDict(
         default="r",
         description="In relaxed mode, "
         "the SPF-authenticated "
-        "domain and RFC5322 "
+        "domain and RFC 5322 "
         "From domain must have "
         "the same "
         "Organizational Domain. "
@@ -224,7 +223,7 @@ dmarc_tags = OrderedDict(
     ),
     p=OrderedDict(
         name="Requested Mail Receiver Policy",
-        reqired=True,
+        required=True,
         description="Specifies the policy to "
         "be enacted by the "
         "Receiver at the "
@@ -375,10 +374,42 @@ dmarc_tags = OrderedDict(
         'specified by the "p" '
         "tag MUST be applied "
         "for subdomains.",
+        values={
+            "none": "The Domain Owner requests "
+            "no specific action be "
+            "taken regarding delivery "
+            "of messages.",
+            "quarantine": "The Domain Owner "
+            "wishes to have "
+            "email that fails "
+            "the DMARC mechanism "
+            "check be treated by "
+            "Mail Receivers as "
+            "suspicious. "
+            "Depending on the "
+            "capabilities of the "
+            "MailReceiver, "
+            "this can mean "
+            '"place into spam '
+            'folder", '
+            '"scrutinize '
+            "with additional "
+            'intensity", and/or '
+            '"flag as '
+            'suspicious".',
+            "reject": "The Domain Owner wishes "
+            "for Mail Receivers to "
+            "reject "
+            "email that fails the "
+            "DMARC mechanism check. "
+            "Rejection SHOULD "
+            "occur during the SMTP "
+            "transaction.",
+        },
     ),
     v=OrderedDict(
         name="Version",
-        reqired=True,
+        required=True,
         description="Identifies the record "
         "retrieved as a DMARC "
         "record. It MUST have the "
@@ -400,6 +431,7 @@ def _query_dmarc_record(
     nameservers: list[str] = None,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
     ignore_unrelated_records: bool = False,
 ) -> Union[str, None]:
     """
@@ -411,6 +443,8 @@ def _query_dmarc_record(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                           requests
         timeout (float): number of seconds to wait for a record from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
+        ignore_unrelated_records (bool): Do not raise a warning if unrelated records are found
 
     Returns:
         str: A record string or None
@@ -424,14 +458,19 @@ def _query_dmarc_record(
 
     try:
         records = query_dns(
-            target, "TXT", nameservers=nameservers, resolver=resolver, timeout=timeout
+            target,
+            "TXT",
+            nameservers=nameservers,
+            resolver=resolver,
+            timeout=timeout,
+            timeout_retries=timeout_retries,
         )
         for record in records:
             if record.startswith(txt_prefix):
                 dmarc_records.append(record)
             elif record.strip().startswith(txt_prefix):
                 raise DMARCRecordStartsWithWhitespace(
-                    "Found a DMARC record that starts with whitespace. "
+                    f"Found a DMARC record at {target} that starts with whitespace. "
                     "Please remove the whitespace, as some implementations "
                     "may not process it correctly."
                 )
@@ -463,6 +502,7 @@ def _query_dmarc_record(
                 nameservers=nameservers,
                 resolver=resolver,
                 timeout=timeout,
+                timeout_retries=timeout_retries,
             )
             for record in records:
                 if record.startswith(txt_prefix):
@@ -485,7 +525,7 @@ def _query_dmarc_record(
     except MultipleDMARCRecords as error:
         raise error
     except Exception as error:
-        raise DMARCRecordNotFound(error)
+        raise DMARCError(error)
 
     return dmarc_record
 
@@ -496,6 +536,7 @@ def query_dmarc_record(
     nameservers: list[str] = None,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
     ignore_unrelated_records: bool = False,
 ) -> OrderedDict:
     """
@@ -507,6 +548,7 @@ def query_dmarc_record(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                           requests
         timeout (float): number of seconds to wait for a record from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
         ignore_unrelated_records (bool): Ignore unrelated TXT records
 
     Returns:
@@ -533,6 +575,7 @@ def query_dmarc_record(
             nameservers=nameservers,
             resolver=resolver,
             timeout=timeout,
+            timeout_retries=timeout_retries,
             ignore_unrelated_records=ignore_unrelated_records,
         )
     except DMARCRecordNotFound:
@@ -543,13 +586,18 @@ def query_dmarc_record(
 
     try:
         root_records = query_dns(
-            domain, "TXT", nameservers=nameservers, resolver=resolver, timeout=timeout
+            domain,
+            "TXT",
+            nameservers=nameservers,
+            resolver=resolver,
+            timeout=timeout,
+            timeout_retries=timeout_retries,
         )
         for root_record in root_records:
             if root_record.startswith("v=DMARC1"):
                 warnings.append(f"DMARC record at root of {domain} has no effect.")
     except dns.resolver.NXDOMAIN:
-        raise DMARCRecordNotFound(f"The domain {domain} does not exist.")
+        raise DMARCRecordNotFound("The domain does not exist.")
     except dns.exception.DNSException:
         pass
 
@@ -559,13 +607,17 @@ def query_dmarc_record(
             nameservers=nameservers,
             resolver=resolver,
             timeout=timeout,
+            timeout_retries=timeout_retries,
             ignore_unrelated_records=ignore_unrelated_records,
         )
         location = base_domain
     if record is None:
-        raise DMARCRecordNotFound(
-            "A DMARC record does not exist for this domain or its base domain."
-        )
+        error_str = "A DMARC record does not exist"
+        if domain == base_domain:
+            error_str += "."
+        else:
+            error_str += " for this subdomain or its base domain."
+        raise DMARCRecordNotFound(error_str)
 
     return OrderedDict(
         [("record", record), ("location", location), ("warnings", warnings)]
@@ -666,6 +718,7 @@ def check_wildcard_dmarc_report_authorization(
     ignore_unrelated_records: bool = False,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
 ) -> bool:
     """
     Checks for a wildcard DMARC report authorization record, e.g.:
@@ -681,6 +734,8 @@ def check_wildcard_dmarc_report_authorization(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                           requests
         timeout (float): number of seconds to wait for an answer from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
+
 
     Returns:
         bool: An indicator of the existence of a valid wildcard DMARC report
@@ -696,6 +751,7 @@ def check_wildcard_dmarc_report_authorization(
             nameservers=nameservers,
             resolver=resolver,
             timeout=timeout,
+            timeout_retries=timeout_retries,
         )
 
         for record in records:
@@ -730,10 +786,11 @@ def verify_dmarc_report_destination(
     ignore_unrelated_records: bool = False,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
 ) -> None:
     """
     Checks if the report destination accepts reports for the source domain
-    per RFC 7489, section 7.1. Raises
+    per RFC 7489, § 7.1. Raises
     `checkdmarc.dmarc.UnverifiedDMARCURIDestination` if it doesn't accept.
 
     Args:
@@ -744,6 +801,8 @@ def verify_dmarc_report_destination(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                         requests
         timeout (float): number of seconds to wait for an answer from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
+
 
     Raises:
         :exc:`checkdmarc.dmarc.UnverifiedDMARCURIDestination`
@@ -759,6 +818,8 @@ def verify_dmarc_report_destination(
             nameservers=nameservers,
             ignore_unrelated_records=ignore_unrelated_records,
             resolver=resolver,
+            timeout=timeout,
+            timeout_retries=timeout_retries,
         ):
             return
         target = f"{source_domain}._report._dmarc.{destination_domain}"
@@ -778,6 +839,7 @@ def verify_dmarc_report_destination(
                 nameservers=nameservers,
                 resolver=resolver,
                 timeout=timeout,
+                timeout_retries=timeout_retries,
             )
 
             for record in records:
@@ -812,6 +874,7 @@ def parse_dmarc_record(
     ignore_unrelated_records: bool = False,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
     syntax_error_marker: str = SYNTAX_ERROR_MARKER,
 ) -> OrderedDict:
     """
@@ -827,6 +890,7 @@ def parse_dmarc_record(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                           requests
         timeout (float): number of seconds to wait for an answer from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
         syntax_error_marker (str): The maker for pointing out syntax errors
 
     Returns:
@@ -905,6 +969,8 @@ def parse_dmarc_record(
     tags["p"]["value"] = tags["p"]["value"].lower()
     if "sp" not in tags:
         tags["sp"] = OrderedDict([("value", tags["p"]["value"]), ("explicit", False)])
+    # Normalize sp value for validation consistency (mirrors p behavior)
+    tags["sp"]["value"] = tags["sp"]["value"].lower()
     if list(tags.keys())[1] != "p":
         raise DMARCSyntaxError("the p tag must immediately follow the v tag.")
     tags["v"]["value"] = tags["v"]["value"].upper()
@@ -919,11 +985,11 @@ def parse_dmarc_record(
             allowed_values = dmarc_tags[tag]["values"]
         if tag == "p" and tag_value == "none":
             warnings.append(
-                f"A p tag value of none has no effect on email sent as {domain}."
+                f"A p tag value of none makes DMARC unenforced on email sent as {domain}."
             )
         if tag == "sp" and tag_value == "none" and explicit:
             warnings.append(
-                f"An sp tag value of none has no effect on email sent as a subdomain of {domain}."
+                f"An sp tag value of none makes DMARC unenforced on email sent as a subdomain of {domain}."
             )
         if tag == "fo":
             tag_value = tag_value.split(":")
@@ -982,6 +1048,7 @@ def parse_dmarc_record(
                         ignore_unrelated_records=ignore_unrelated_records,
                         resolver=resolver,
                         timeout=timeout,
+                        timeout_retries=timeout_retries,
                     )
                 try:
                     hosts = get_mx_records(
@@ -989,6 +1056,7 @@ def parse_dmarc_record(
                         nameservers=nameservers,
                         resolver=resolver,
                         timeout=timeout,
+                        timeout_retries=timeout_retries,
                     )
                     if len(hosts) == 0:
                         raise DMARCReportEmailAddressMissingMXRecords(
@@ -1043,6 +1111,7 @@ def parse_dmarc_record(
                         ignore_unrelated_records=ignore_unrelated_records,
                         resolver=resolver,
                         timeout=timeout,
+                        timeout_retries=timeout_retries,
                     )
                 try:
                     hosts = get_mx_records(
@@ -1050,6 +1119,7 @@ def parse_dmarc_record(
                         nameservers=nameservers,
                         resolver=resolver,
                         timeout=timeout,
+                        timeout_retries=timeout_retries,
                     )
                     if len(hosts) == 0:
                         raise DMARCReportEmailAddressMissingMXRecords(
@@ -1117,6 +1187,7 @@ def get_dmarc_record(
     nameservers: list[str] = None,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
 ) -> OrderedDict:
     """
     Retrieves a DMARC record for a domain and parses it
@@ -1128,6 +1199,7 @@ def get_dmarc_record(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                           requests
         timeout (float): number of seconds to wait for an answer from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
 
     Returns:
         OrderedDict: An ``OrderedDict`` with the following keys:
@@ -1162,6 +1234,7 @@ def get_dmarc_record(
         nameservers=nameservers,
         resolver=resolver,
         timeout=timeout,
+        timeout_retries=timeout_retries,
     )
 
     return OrderedDict(
@@ -1178,6 +1251,7 @@ def check_dmarc(
     nameservers: list[str] = None,
     resolver: dns.resolver.Resolver = None,
     timeout: float = 2.0,
+    timeout_retries: int = 2,
 ) -> OrderedDict:
     """
     Returns a dictionary with a parsed DMARC record or an error
@@ -1191,6 +1265,8 @@ def check_dmarc(
         resolver (dns.resolver.Resolver): A resolver object to use for DNS
                                           requests
         timeout (float): number of seconds to wait for a record from DNS
+        timeout_retries (int): The number of times to reattempt a query after a timeout
+
 
     Returns:
         OrderedDict: An ``OrderedDict`` with the following keys:
@@ -1214,6 +1290,7 @@ def check_dmarc(
             nameservers=nameservers,
             resolver=resolver,
             timeout=timeout,
+            timeout_retries=timeout_retries,
         )
         dmarc_results["record"] = dmarc_query["record"]
         dmarc_results["location"] = dmarc_query["location"]
@@ -1226,6 +1303,7 @@ def check_dmarc(
             nameservers=nameservers,
             resolver=resolver,
             timeout=timeout,
+            timeout_retries=timeout_retries,
         )
         dmarc_results["warnings"] = dmarc_query["warnings"]
 
